/**
 * Search for sender code for Thunderbird
 *
 * Thunderbird V3
 *
 * LICENSE: This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @author        Einar Egilsson
 * @author        Fabien Arcellier <fabien_arcellier@yahoo.fr>
 * @license       http://www.gnu.org/licenses/lgpl-3.0.txt LGPL
 * @version       $Revision$
 * @lastrevision  $Date$
 * @lastmodified  $LastChangedDate$
 **/

/**
* Root package
**/
var SearchForSender = {
    
    /**
     * Function binded to the load event of thunderbird window
     *
     * 
     * @param event It does not use this variable, it's a function requirement
     * @exemple 
     * window.addEventListener("load", function(event) { SearchForSender.onLoad(event); }, false);
     **/ 
    onLoad : function(event) {
        document.getElementById('mailContext').
          addEventListener("popupshowing", function(e) { SearchForSender.showContextMenu(e); }, false);
    },
    
    /**
     * Function binded to the context menu loading
     *
     * Thanks to this function, we filter the content of context menu when we
     * right click on messages.
     * - If we have only one message selected, we show search for sender 
     *  actions (search for sender, search for subject...)
     * - If we have many messages selected, it hides the search for sender actions
     *
     * @param event It does not use this variable, it's a function requirement
     * @exemple
     * document.getElementById('mailContext').
     *    addEventListener("popupshowing", function(e) { SearchForSender.showContextMenu(e); }, false);
     **/ 
    showContextMenu : function(event) {
        var hide = false;
        hide = GetNumSelectedMessages() != 1;
        var sfe_searchSubjectNode = document.getElementById("searchforsender-subject");
        var sfe_searchSenderNode = document.getElementById("searchforsender-sender");
        var sfe_SeparatorNode = document.getElementById("searchforsender-sep");
        sfe_searchSubjectNode.hidden = hide;
        sfe_searchSenderNode.hidden = hide;
        sfe_SeparatorNode.hidden = hide;
    },
    
    /**
     * Get the subject of first message in the selection and use it as search value
     *
     * @param event This variable is not used, it's a function requirement
     * @exemple
     * SearchForSender.searchForSubject(event)
     **/
    searchForSubject: function(event) {
        var indiceMessage = gFolderDisplay.selectedIndices[0];        
        var msgHdr = gDBView.getMsgHdrAt(indiceMessage);
        var subject = msgHdr.mime2DecodedSubject.toString();
        subject = subject.replace(/^(Re:|fwd:|fw:|spam-low:|out of office autoreply:|\s|\[)*/i, "");
        subject = subject.replace(/(\s|\])*$/i, "");
        this.search(subject, false, true);
    },
    
    /**
     * Get the sender of the first message in the selection and use it as search value
     *
     * @param event It does not use this variable, it's a function requirement
     * @exemple
     * SearchForSender.searchForSender(event)
     **/
    searchForSender: function(event) {
        var indiceMessage = gFolderDisplay.selectedIndices[0];
        var msgHdr = gDBView.getMsgHdrAt(indiceMessage);
        var parser = Components.classes["@mozilla.org/messenger/headerparser;1"].
          getService(Components.interfaces.nsIMsgHeaderParser);
        var senderAddress = {};
        parser.parseHeadersWithArray(msgHdr.author, senderAddress, {}, {});
        var sender = senderAddress.value.toString();
        this.search(sender, true, false);
    },

    /**
     * Execute a research using the mode Filter using the subject or the sender
     *
     * 
     * @param string searchTerm string the script set in the filter's textbox
     * @param bool useSenderAsCriteria Boolean to param the filter's behavior.
     *    If true, it compares filter textbox's content with sender
     *    If false, it do not compare filter textbox's content with sender
     * @param bool useSubjectAsCriteria Boolean to param the filter's behavior
     *    If true, it compares filter textbox's content with subject
     *    If false, it do not compare filter textbox's content with subject
     * @exemple
     * SearchForSender.searchForSender("@yahoo.fr", true, false) //Display only mails sent from the domain yahoo.fr
     * @see mail\base\content\quickFilterBar.xul in TB source
     **/
    search : function(searchTerm, useSenderAsCriteria, useSubjectAsCriteria) {
        var filterTextBox = document.getElementById("qfb-qs-textbox");
        var useSenderAsCriteriaCheckbox = document.getElementById("qfb-qs-sender");
        var useRecipientsAsCriteriaCheckbox = document.getElementById("qfb-qs-recipients");
        var useSubjectAsCriteriaCheckbox = document.getElementById("qfb-qs-subject");
        var useBodyAsCriteriaCheckbox = document.getElementById("qfb-qs-body");
        
        //precondition
        SearchForSender.debug.assert(typeof searchTerm === "string", "search : method search, searchTerm is not a string - Type : " + typeof searchTerm + " - Value : " + searchTerm);
        SearchForSender.debug.assert(filterTextBox !== null, "search : The filter text box qfb-qs-textbox is missing");
        SearchForSender.debug.assert(useSenderAsCriteriaCheckbox !== null, "search : The checkbox qfb-qs-sender is missing");
        SearchForSender.debug.assert(useRecipientsAsCriteriaCheckbox !== null, "search : The checkbox qfb-qs-recipients is missing");
        SearchForSender.debug.assert(useSubjectAsCriteriaCheckbox !== null, "search : The checkbox qfb-qs-subject is missing");
        SearchForSender.debug.assert(useBodyAsCriteriaCheckbox !== null, "search : The checkbox qfb-qs-body is missing");
        SearchForSender.debug.assert((useSenderAsCriteria === false && useSubjectAsCriteria === false) === false, "search : useSenderAsCriteria and useSubjectAsCriteria are both setting to false, search method can not find anything");
        
        //function
        if(filterTextBox !== null)
        {
          this.setCheckBoxValue(useSenderAsCriteriaCheckbox, useSenderAsCriteria);
          this.setCheckBoxValue(useRecipientsAsCriteriaCheckbox, false);
          this.setCheckBoxValue(useSubjectAsCriteriaCheckbox, useSubjectAsCriteria);
          this.setCheckBoxValue(useBodyAsCriteriaCheckbox, false);         
          filterTextBox.value = searchTerm;
          filterTextBox.doCommand();
        }
        
    },
    
    /**
     * Active or unactive a XUL checkbox
     * 
     * This function is useful because there are two attributes, checked and checkState
     * to modify when you would like to change the checkbox status properly.
     *  
     * @param checkbox checkBox a XUL element with the type checkbox
     * @param bool status boolean value to active or unactive the checkbox
     *
     * @exemple
     * SearchForSender.setCheckBoxValue(document.getElementById("qfb-qs-sender"), false);
     **/
    setCheckBoxValue: function(checkBox, status)
    {
      //precondition
      SearchForSender.debug.assert(checkBox !== null, "setCheckBoxValue :checkBox is null");
      SearchForSender.debug.assert(checkBox.type === "checkbox", "setCheckBoxValue : checkBox has not the type checkbox");
      SearchForSender.debug.assert(status === true || status === false,"setCheckBoxValue : status is not a boolean");
      
      //function
      checkBox.checked = status;
      if(status === true)
      {
        checkBox.checkState = 1;
      } 
      else
      {
        checkBox.checkState = 0;
      }
      
    }
};

 /**
  * Debug methods. On the production environment methods of this package are
  * unactives.
  *
  */
SearchForSender.debug = {

  /**
   * If debug member is set at true, this extension is in debug mode.
   *
   * When the debug mode is active, Exceptions and error messages are logged in
   * Thunderbird console.
   * It also logs execution messages trigged at key steps and some assertions
   * to verify the result in conform to the logic we have specified.
   **/
  debug: false,
  
  /**
   * Get console service from Thunderbird
   * 
   **/
  getConsole: function()
  {
    var consoleServiceComponent = Components.classes["@mozilla.org/consoleservice;1"];
    return consoleServiceComponent.getService(Components.interfaces.nsIConsoleService);
  },
  
  /**
   * Display a message on the console when debug mode is active
   *
   * @param aMessage array Messages to display
   */
  dump: function(aMessage)
  {
    if(this.debug === true)
    {
      var consoleService = this.getConsole();
      consoleService.logStringMessage("Search for sender : " + aMessage);
    }
    
  },//end dump()
  
  /**
   * Assert expression. If expression return true, a message is displayed in
   * Thunderbird console.
   * This function is only active when debug mode is turn to (debug member = true)
   *
   */
  assert: function(exp, message)
  {
    if(this.debug === true && exp === false)
    {
      var consoleService = this.getConsole();
      consoleService.logStringMessage("Search for sender : " + message);
    }
    
  }//end assert()
  
};//end SearchForSender.debug{}

window.addEventListener("load", function(event) { SearchForSender.onLoad(event); }, false);